\ ----------------------------------------------------------------------
\ LANGUAGE    : AVR ByteForth vsn 2.00
\ PROJECT     : Ushi mijn robot belofte voor de HCC Forth gg
\ DESCRIPTION : Ushi-3 als kleine autonome robot 4 MHz versie
\ CATEGORY    : Applicatie, afmeting: 1218 bytes.
\ AUTHOR      : Willem Ouwerkerk, Februari 07, 2001
\ LAST CHANGE : Willem Ouwerkerk, August 18, 2003
\ ----------------------------------------------------------------------

90S2313                             \ Selecteer target
NEEDS TARGET.FRT                    \ Laad cpu specifiek deel
7 8 8 MEMORY                        \ Strip geheugengebruik
4 SET-CRYSTAL                       \ Gebruik 4 MHz kristal
NEEDS MS.FRT                        \ Laad geschikte MS versie

INCLUDE USHIO-1.FRT                 \ Laad basis Ushi-I/O routines


DOC over het aansluiten van de kabeltjes

Deze tabel geldt voor Ushi met een 90S2313 of 89Cx051

 1 - GP2 data           22 - NC
 2 - GP2 control        21 - NC
 3 - IR zender          20 - NC
 4 - IR ontvanger       19 - Bumper linksvoor
 5 - NC                 18 - Bumper linksopzij
 6 - NC                 17 - Vloer links
 7 - NC                 16 - NC
 8 - Reserve            15 - Bumper achter
 9 - Vloer achter       14 - Bumper rechtsvoor
10 - Motor links        13 - Bumper rechtsopzij
11 - Motor rechts       12 - Vloer rechts

ENDDOC


\ Wachtlussen

: 100MS             ( tijd -- )
    FOR  100 MS  NEXT
    ;

\ Wacht even maar stop als de tafelrand gezien is of de
\ bumper aan de achterkant ingedrukt is.
: /SEC              ( tijd -- )
    FOR
        TAFELRAND?  ACHTER 0=  OR
        IF  UNNEXT  STOP EXIT  THEN  100 MS
    NEXT
    ;

: SEC               ( sec -- )
    FOR  10 /SEC  NEXT
    ;

: DRAAI             ( u*100ms -- )
    2 CHOOSE IF  RECHTSOM  ELSE  LINKSOM  THEN  /SEC
    ;


\ Maak ontwijk beweging van u maal 100 millisec. naar links of rechts.
: KEEROM            ( u*100ms -- )
    ACHTERUIT 1 SEC  DRAAI
    ;

\ Rij niet van de tafel af !!
: DRAAIWEG          ( -- )
    TAFELRAND  BEGIN-SELECT
        SELECT 6  ACHTERUIT  4 100MS  8 KEEROM EXIT
        SELECT 4  ACHTERUIT  8 100MS  LINKS EXIT
        SELECT 2  ACHTERUIT  8 100MS  RECHTS EXIT
        SELECT 1  VOORUIT EXIT
    END-SELECT
    ;

\ Probeer bij de tafelrand weg te komen, lukt dat niet, zet de 
\ robot dan stil. Hier wordt een kale vertragings-routine
\ gebruikt omdat er een tafelrand stop in /SEC en SEC gebouwd zit.
: ONTSNAP           ( -- )
    DRAAIWEG  10 100MS
    TAFELRAND? IF  DRAAIWEG  6 100MS  THEN
    BEGIN  TAFELRAND? WHILE  STOP  REPEAT
    ;

\ Maak een uitwijk beweging na je te hebben gestoten.
: ONTWIJK           ( -- )
    GESTOTEN 16 =   IF  VOORUIT EXIT    THEN
    GESTOTEN  5 =   IF  15 KEEROM EXIT  THEN
    GESTOTEN  3 AND IF  LINKSOM EXIT    THEN
    GESTOTEN 12 AND IF  RECHTSOM EXIT   THEN
    ;

: BEWEEG            ( -- )
    TAFELRAND? IF  ONTSNAP EXIT  THEN
    GESTOTEN? IF  ONTWIJK  10 /SEC EXIT  THEN
    AFSTAND 80 U< IF  10 DRAAI EXIT  THEN
    VOORUIT
    ;

: HARDER            ( -- )
    FROM TEMPO 2 +  SNELHEID
    ;

: ZACHTER           ( -- )
    FROM TEMPO 2 -  0 MAX SNELHEID
    ;

FLAG STOP?

: START/STOP        ( -- )
    STOP? IF  STOP  ELSE  VOORUIT  THEN  TOGGLE STOP?
    ;

: SKIP-RCKEY        ( -- )
    150 MS  RCKEY? IF  RCKEY DROP  THEN
    ;

: AUTONOOM          ( -- )
    BEGIN  BEWEEG  RCKEY? UNTIL
    ;

: AFSTANDBEDIENING  ( -- )
    RCKEY  SKIP-RCKEY
    BEGIN-SELECT
        SELECT 12  AUTONOOM EXIT        \ Aan/Uit toets RC5
        SELECT 13  START/STOP EXIT      \ Geluid aan/uit RC5
        SELECT 16  HARDER EXIT          \ Volume + RC5
        SELECT 17  ZACHTER EXIT         \ Volume - RC5
        SELECT 32  RECHTSOM EXIT        \ Kanaal + RC5
        SELECT 33  LINKSOM EXIT         \ Kanaal - RC5
    END-SELECT
    ;

\ Leef een wild en genadeloos leven maar wacht
\ eerst op toestemming daartoe.
: LEEF              ( -- )
    SETUP  SETUP-USHI  40 SNELHEID  STOP
    BEGIN
        RCKEY? IF  AFSTANDBEDIENING  THEN
        TAFELRAND? IF  STOP  THEN
    AGAIN
    ;  MAIN

                              ( Einde )
